(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: DMZDatatype.thy 8881 2010-04-02 09:37:05Z brucker $ *)

theory 
  DMZDatatype
imports 
  FWTesting
begin

profiling_on

text{* This is the fourth scenario, slightly more complicated than the
previous one, as we now also model specific servers within one
network. Therefore, we could not use anymore the modelling using
datatypes, but only use the one where an address is modelled as an
integer (with ports). 

Just for comparison, this theory is the same scenario with datatypes
anyway, but with four distinct networks instead of one contained in
another. As there is no corresponding network model included, we need
to define a custom one.  *}


datatype Adr = Intranet | Internet | Mail | Web | DMZ
instance Adr::adr ..
types port = int
types Networks = "Adr \<times> port"


constdefs
  intranet::"Networks net"
 "intranet \<equiv> {{(a,b). a= Intranet}}"
  dmz :: "Networks net"
 "dmz \<equiv> {{(a,b). a= DMZ}}"
  mail :: "Networks net"
 "mail \<equiv> {{(a,b). a=Mail}}"
  web :: "Networks net"
 "web \<equiv> {{(a,b). a=Web}}"
  internet :: "Networks net"   
 "internet \<equiv> {{(a,b). a= Internet}}"

defs (overloaded)
 src_port_def:  "src_port (x::(Networks,'b) packet)  \<equiv> (snd o fst o snd) x"
 dest_port_def: "dest_port (x::(Networks,'b) packet)  \<equiv> (snd o fst o snd o snd) x"
 subnet_of_def: "subnet_of (x::Networks) \<equiv> {{(a,b). a = fst x}}" 


constdefs 
  Intranet_mail_port :: "(Networks ,DummyContent) Policy"
  "Intranet_mail_port \<equiv> (allow_from_ports_to {21::port,14} intranet mail)" 

  Intranet_Internet_port :: "(Networks,DummyContent) Policy"
  "Intranet_Internet_port \<equiv> allow_from_ports_to {80::port,90} intranet internet"
     
  Internet_web_port :: "(Networks,DummyContent) Policy"
  "Internet_web_port \<equiv> (allow_from_ports_to {80::port,90} internet web)"
          
  Internet_mail_port :: "(Networks,DummyContent) Policy"
  "Internet_mail_port \<equiv>  (allow_all_from_port_to internet (21::port) dmz)"

definition 
  policyPort :: "(Networks, DummyContent) Policy" where
 "policyPort \<equiv> deny_all ++ 
                  Intranet_Internet_port ++ 
                  Intranet_mail_port ++  
                  Internet_mail_port ++ 
                  Internet_web_port"


text {* We only want to create test cases which are sent between the
three main networks --- e.g. not between the mailserver and the
dmz. Therefore, the constraint looks as follows. *}

definition
 not_in_same_net :: "(Networks,DummyContent) packet \<Rightarrow> bool" where
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                        (src x \<sqsubset> intranet \<longrightarrow> \<not> dest x \<sqsubset> intranet) \<and> 
                        (src x \<sqsubset> dmz \<longrightarrow> \<not> dest x \<sqsubset> dmz)"

lemmas PolicyLemmas =  policyPort_def dmz_def internet_def intranet_def mail_def web_def 
                   
                       Internet_web_port_def Internet_mail_port_def
                       Intranet_Internet_port_def Intranet_mail_port_def 
                       Internet_web_port_def Internet_mail_port_def

                       src_def dest_def src_port_def dest_port_def in_subnet_def

test_spec "not_in_same_net x \<longrightarrow> FUT (x:: (Networks,DummyContent) packet) = policyPort x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas PortCombinators PolicyCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)
store_test_thm "port"

gen_test_data "port"
thm port.test_data

write_clocks "time_stats_DMZDatatype.tex"

profiling_off



end
